==> mailfront-plugin-remotehost-0.01.tar.gz: mailfront-plugin-remotehost-0.01/plugin-remotehost.c <==
#include "mailfront.h"

#include <sysdeps.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <msg/msg.h>
#include <stdlib.h>
#include <str/str.h>

#include <sys/types.h>
#include <regex.h>
#include <stdio.h>

static unsigned linemax = 512;

static response resp_remotehost = { 0, 0 };

/* Below function copied from http://n.h7a.org/hacks/qmail-badremotehost.patch */
/* Another is qregex.c/qregex.h aka qmail-regex but I don't think I need it here. :p */
int is_regex_matched(char *regex, char *text)
{
	int rc;
	regex_t reg;

	rc = regcomp(&reg,regex,REG_EXTENDED|REG_ICASE);
	if (!rc) rc = regexec(&reg,text,0,0,0);
	regfree(&reg);

	return rc ? 0 : 1;
}

static const response* data_start(void)
{
	/* I not good in C and instead of spending time to read, learn and use bglibs I think I will try to use basic C functions that I know a little :p  Sorry, guys! */
	FILE *controlfile;
	const char* tmp;
	unsigned u;
	char *rhc;		/* for TCPREMOTEHOST */
	int len, msglen, i;
	int wlmatch = 0, blmatch = 0, lineno;
	char *badremotehost, *goodremotehost;
	char *defaultmsg = "Sorry, your remotehost match our prohibited list. (#5.7.1)";
	unsigned long pid;
	const char *ip;

	/* if RELAYCLIENT is set or authenticated user then we skip remotehost */
	if ((session_getenv("RELAYCLIENT") != 0) || session_getnum("authenticated", 0) ) {
		//warn1("remotehost plugin skipped: relayclient/authenticated user");
		return 0;
	}

	/* Get linemax default to 256 */
	if ((tmp = session_getenv("REMOTEHOST_LINEMAX")) != 0) {
		if ((u = strtoul(tmp, (char**)&tmp, 10)) > 0 && *tmp == 0)
			linemax = u;
	}
	char myline[linemax];
	char rejectmsg[linemax];

	/* get IP from environment variable TCPREMOTEIP */
	if (session_getenv("TCPREMOTEIP") != 0) {
		ip = session_getenv("TCPREMOTEIP");
	} else {
		ip = "";
	}

	/* If TCPREMOTEHOST is empty or not set then we get TCPREMOTEIP value instead */
	rhc = session_getenv("TCPREMOTEHOST");
	if (!rhc) rhc = ip;

	/* get pid */
	pid = (unsigned long)getpid();

	/* check whitelist */
	goodremotehost = session_getenv("GOODREMOTEHOST");
	//if (!goodremotehost) warn1("remotehost plugin: GOODREMOTEHOST environment not set.");
	if (goodremotehost) {
		controlfile = fopen(goodremotehost, "r");
		if (controlfile == NULL) {
			fprintf(stderr, "mailfront[%lu]: remotehost plugin: Could not read goodremotehost file %s\n", pid, goodremotehost);
		}
		else {
			lineno = 0;
			/* getline is preferred if include #define _GNU_SOURCE but is not compatible with freebsd since this is not standard C library */
			while(fgets(myline, linemax, controlfile) != NULL) {
				lineno++;
				if (myline[0] != '#') {
					/* replace newline with null */
					len = strlen(myline);
					if (myline[len-1] == '\n')
						myline[len-1] = '\0';
					if (is_regex_matched(myline, rhc)) {
						wlmatch++;
						fprintf(stderr, "mailfront[%lu]: remotehost plugin: %s whitelist matched at line %d: %s\n", pid, ip, lineno, rhc);
					}
				}
			}
			fclose(controlfile);
		}
	}
	if (wlmatch > 0) {
		warn1("remotehost whitelist found");
		return 0;
	}

	/* check black list */
	badremotehost = session_getenv("BADREMOTEHOST");
	//if (!badremotehost) warn1("remotehost plugin: BADREMOTEHOST environment not set.");
	if (badremotehost) {
		controlfile = fopen(badremotehost, "r");
		if (controlfile == NULL) {
			fprintf(stderr, "mailfront[%lu]: remotehost plugin: Could not read badremotehost file %s\n", pid, badremotehost);
		}
		else {
			lineno = 0;
			while(fgets(myline, linemax, controlfile) != NULL) {
				lineno++;
				/* replace newline with null */
				len = strlen(myline);
				if (myline[len-1] == '\n')
					myline[len-1] = '\0';
				if (myline[0] != '#') {
					if (is_regex_matched(myline, rhc)) {
						blmatch++;
						fprintf(stderr, "mailfront[%lu]: remotehost plugin: %s blacklist matched at line %d: %s\n", pid, ip, lineno, rhc);
					}
				}
				else {
					if (myline[1] == '=') {
						if (blmatch == 0) {
							/* Don't know whether the below method is suitable but it works */
							for (i=2; i<=strlen(myline); i++) {
								msglen = i - 2;
								rejectmsg[msglen] = myline[i];
							}
							//rejectmsg[msglen+1] = '\0';
						}
					}
				}
			}
			fclose(controlfile);
		}
	}
	if (blmatch > 0) {
		if (session_getenv("REMOTEHOST_REJECT"))
			resp_remotehost.number = 554;
		else
			resp_remotehost.number = 451;
		if (strlen(rejectmsg) > 0)
			resp_remotehost.message = rejectmsg;
		else
			resp_remotehost.message = defaultmsg;
		return &resp_remotehost;
	}

	return 0;
}


struct plugin plugin = {
  .version     = PLUGIN_VERSION,
  .data_start = data_start,
};

==> mailfront-plugin-remotehost-0.02.tar.gz: mailfront-plugin-remotehost-0.02/plugin-remotehost.c <==
#include "mailfront.h"

#include <sysdeps.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <msg/msg.h>
#include <stdlib.h>
#include <str/str.h>

#include <sys/types.h>
#include <regex.h>
#include <stdio.h>

static unsigned linemax = 512;

static response resp_remotehost = { 0, 0 };

/* Below function copied from http://n.h7a.org/hacks/qmail-badremotehost.patch */
/* Another is qregex.c/qregex.h aka qmail-regex but I don't think I need it here. :p */
int is_regex_matched(char *regex, char *text)
{
	int rc;
	regex_t reg;

	rc = regcomp(&reg,regex,REG_EXTENDED|REG_ICASE);
	if (!rc) rc = regexec(&reg,text,0,0,0);
	regfree(&reg);

	return rc ? 0 : 1;
}

static const response* data_start(void)
{
	/* I not good in C and instead of spending time to read, learn and use bglibs I think I will try to use basic C functions that I know a little :p  Sorry, guys! */
	FILE *controlfile;
	const char* tmp;
	unsigned u;
	char *rhc;		/* for TCPREMOTEHOST */
	int len, msglen, i;
	int wlmatch = 0, blmatch = 0, lineno;
	char *badremotehost, *goodremotehost;
	char *defaultmsg = "Sorry, your remotehost match our prohibited list. (#5.7.1)";
	unsigned long pid;
	const char *ip;

	/* if RELAYCLIENT is set or authenticated user then we skip remotehost */
	if ((session_getenv("RELAYCLIENT") != 0) || session_getnum("authenticated", 0) ) {
		//warn1("remotehost plugin skipped: relayclient/authenticated user");
		return 0;
	}

	/* Get linemax default to 256 */
	if ((tmp = session_getenv("REMOTEHOST_LINEMAX")) != 0) {
		if ((u = strtoul(tmp, (char**)&tmp, 10)) > 0 && *tmp == 0)
			linemax = u;
	}
	char myline[linemax];
	char rejectmsg[linemax];

	/* get IP from environment variable TCPREMOTEIP */
	if (session_getenv("TCPREMOTEIP") != 0) {
		ip = session_getenv("TCPREMOTEIP");
	} else {
		ip = "";
	}

	/* If TCPREMOTEHOST is empty or not set then we get TCPREMOTEIP value instead */
	rhc = session_getenv("TCPREMOTEHOST");
	if (!rhc) rhc = ip;

	/* get pid */
	pid = (unsigned long)getpid();

	/* check whitelist */
	goodremotehost = session_getenv("GOODREMOTEHOST");
	//if (!goodremotehost) warn1("remotehost plugin: GOODREMOTEHOST environment not set.");
	if (goodremotehost) {
		controlfile = fopen(goodremotehost, "r");
		if (controlfile == NULL) {
			fprintf(stderr, "mailfront[%lu]: remotehost plugin: Could not read goodremotehost file %s\n", pid, goodremotehost);
		}
		else {
			lineno = 0;
			/* getline is preferred if include #define _GNU_SOURCE but is not compatible with freebsd since this is not standard C library */
			while(fgets(myline, linemax, controlfile) != NULL) {
				lineno++;
				if (myline[0] != '#') {
					/* replace newline with null */
					len = strlen(myline);
					if (myline[len-1] == '\n')
						myline[len-1] = '\0';
					/* skip starting line that is null */
					if (is_regex_matched(myline, rhc) && myline[0] != '\0') {
						wlmatch++;
						fprintf(stderr, "mailfront[%lu]: remotehost plugin: %s whitelist matched at line %d: %s\n", pid, ip, lineno, rhc);
					}
				}
			}
			fclose(controlfile);
		}
	}
	if (wlmatch > 0) {
		warn1("remotehost whitelist found");
		return 0;
	}

	/* check black list */
	badremotehost = session_getenv("BADREMOTEHOST");
	//if (!badremotehost) warn1("remotehost plugin: BADREMOTEHOST environment not set.");
	if (badremotehost) {
		controlfile = fopen(badremotehost, "r");
		if (controlfile == NULL) {
			fprintf(stderr, "mailfront[%lu]: remotehost plugin: Could not read badremotehost file %s\n", pid, badremotehost);
		}
		else {
			lineno = 0;
			while(fgets(myline, linemax, controlfile) != NULL) {
				lineno++;
				/* replace newline with null */
				len = strlen(myline);
				if (myline[len-1] == '\n')
					myline[len-1] = '\0';
				/* skip starting line that is null and # */
				if (myline[0] != '#') {
					if (is_regex_matched(myline, rhc) && myline[0] != '\0') {
						blmatch++;
						fprintf(stderr, "mailfront[%lu]: remotehost plugin: %s blacklist matched at line %d: %s\n", pid, ip, lineno, rhc);
					}
				}
				else {
					if (myline[1] == '=') {
						if (blmatch == 0) {
							/* Don't know whether the below method is suitable but it works */
							for (i=2; i<=strlen(myline); i++) {
								msglen = i - 2;
								rejectmsg[msglen] = myline[i];
							}
							//rejectmsg[msglen+1] = '\0';
						}
					}
				}
			}
			fclose(controlfile);
		}
	}
	if (blmatch > 0) {
		if (session_getenv("REMOTEHOST_REJECT"))
			resp_remotehost.number = 554;
		else
			resp_remotehost.number = 451;
		if (strlen(rejectmsg) > 0)
			resp_remotehost.message = rejectmsg;
		else
			resp_remotehost.message = defaultmsg;
		return &resp_remotehost;
	}

	return 0;
}


struct plugin plugin = {
  .version     = PLUGIN_VERSION,
  .data_start = data_start,
};
